/*
---------------------------------------------------------------------------
 SPPP - Programador PIC de Porta Serial para Linux               22/08/2020


 ALGUMAS DEFINICOES

 Software ............... Este programa (sppp) que roda no PC (Linux).
 Firmware ............... Programa que roda no microcontrolador do
                          circuito programador.
 Programador ............ Hardware do circuito programador.
 Alvo ................... Microcontrolador a ser programado usando o
                          circuito programador.

 DESCRICAO GERAL DESTE SOFTWARE

 Este software faz parte de um conjunto formado por um circuito programador
 (hardware) controlado originalmente por um microcontrolador PIC16F648A
 no qual o firmware realiza as funcoes de gravacao, apagamento, listagem
 e outros controles para um microcontrolador alvo.

 Este software foi desenvolvido para a operar numa console do sistema
 operacional Linux usando a linha de comandos para fornecer as opcoes
 desejadas para realizar a tarefa.

 A linha de comandos deve apresentar a seguinte estrutura:

         sppp <device> <options> <filename>

 sppp ..................... nome deste programa
 <porta> .................. algo como /dev/ttyUSBx
 <opcoes> ................. opcao (ver opcoes abaixo)

 Opcoes:
 -de    - Dump da memoria EEPROM.
 -dc    - Dump da memoria de configuracao.
 -dp    - Dump da memoria de programa.
 -ee    - Apaga memoria EEPROM.
 -ep    - Apaga memoria de programa e de configuracao.
 -pic   - Especifica o nome do chip alvo:
          --> sppp /dev/ttyUSB0 -pic pic12f675
 -wp    - Requisita a gravacao do arquivo '.hex' no chip alvo.
          --> sppp /dev/ttyUSB0 -pic pic16F628A -wp ledpisca.hex
 -vd    - Verifica a tensao de alimentacao acendendo o LED vermelho.
 -vp    - Verifica a tensao de programacao acendendo o LED azul.
 -v     - Informacao sobre o chip alvo.


 COMUNICACAO COM O CIRCUITO PROGRAMADOR

 A comunicacao com o circuito programador sera feita atraves de uma porta
 serial do sistema operacional.
 Quando um conversor USB-Serial for utilizado para a comunicacao e este
 conversor fornecer tensao de 5V, o circuito programador podera ser
 alimentado por esta tensao, caso contrario, o circuito programador
 devera se alimentado com uma fonte externa. Deve-se observar que certas
 portas USB podem nao fornecer corrente necessaria para alimentar o circuito
 programador.


 SOBRE O FIRMWARE

 O firmware que roda no programador foi desenvolvido em linguagem assembly
 na plataforma Windows/XP com a IDE MPLAB da Microchip. O projeto completo
 desenvolvido no MPLAB se encontra na pasta '../Hardware/firmware.tar.gz'.


 SOBRE O PROGRAMADOR

 Para poder aproveitar a tensao de saida (5V) fornecida pelos conversores
 USB-Seriais, o hardware do programador usa uma bomba de tensao para
 produzir 13V necessarios para o chip alvo entrar no modo de programacao.
 O hardware do circuito programador foi desenvolvido em torno de um PIC16F648A
 e desenhado no Cad Eagle. O projeto completo se encontra na pasta
 '../Hardware/pcb.tar.gz'.


 EXEMPLOS DE USO

 Para listar um Dump da memoria EEPROM, usar o comando:
 sppp /dev/ttyUSB0 -de

 Para listar um Dump da memoria de programa:
 sppp /dev/ttyUSB0 -dp

 Para gravar o programa no chip alvo:
 sppp /dev/ttyUSB0 -wp programa.hex

 Para apagar a memoria de programa:
 sppp /dev/ttyUSB0 -ep


 LINK SIMBOLICO

 Apos a compilacao pode-se fazer o link simbolico abaixo para executar
 o programa estando o terminal aberto em qualquer pasta do sistema:

 sudo ln -s  ~/SPINPROG/sppp/sppp /usr/bin/sppp

---------------------------------------------------------------------------
*/

/*  VARIAVEIS GLOBAIS */
unsigned int FLAGS=0;           /* flags do programa */
#define  WP_PRESENTE  1         /* indica opcao -wp */


char serialPort[24];            /* device name */
char* programfile;              /* nome do arquivo .hex */
char comando[2];                /* numero do comando a enviar ao firmware */
int  serial_port;               /* terminal file handler */
char read_buf[64];              /* input buffer for programmer response */
char main_buffer[4096];         /* input buffer to hold lot of data */
struct termios tty;             /* definicoes do terminal */
unsigned int picdet;            /* microcontrolador detetado */
unsigned int pgmpic;            /* microcontrolador no programa */
char picid[12];                 /* usado por getPicId() */

/* BIBLIOTECAS DE FUNCOES */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>

#include "Includes/includes.h"


/* FUNCOES LOCAIS - Nao modificar as posicoes desses includes */
/*
#include "help.c"
#include "checkopt.c"
#include "getopt.c"
#include "ascii2hex.c"
#include "getchipid.c"
#include "getpicid.c"
#include "mybits.c"
#include "checkchip.c"
#include "rundumps.c"
#include "runoptiondc.c"
#include "runoptionde.c"
#include "runoptiondp.c"
#include "runoptionee.c"
#include "runoptionep.c"
#include "runoptionwp.c"
#include "runoptionvd.c"
#include "runoptionvp.c"
#include "runoptioninfo.c"
*/



int main(int argc, char *argv[])
{

    printf("SPPP - PROGRAMADOR PIC DE PORTA SERIAL - AGOSTO DE 2020\n");
     /*
     // --------------------------------------------------------------
     // COLOQUE ABAIXO AS OPCOES QUE PODEM RODAR ANTES DA CONFIGURACAO
     // DA PORTA SERIAL.
     // --------------------------------------------------------------

     // --------------------------------------------------------------
     // Verifica se o usuario pediu ajuda (-h --> Help)
     // -------------------------------------------------------------- */
     if( checkOpt("-h", argc, argv) == 0 ) {
        printf("\n");
        help();
        return 0;
     }



     if( argc < 2 ) {
        printf("NADA A FAZER\n");
        printf("ESPECIFIQUE UM DISPOSITIVO SERIAL COMO /dev/ttyUSB1\n");
        printf("OU CONSULTE AS OPCOES ABAIXO:\n\n");
        help();
        printf("\n");
        return 0;
     }


    /* -------------------------------------------------------------
    // Configuracao do terminal
    // ------------------------------------------------------------- */
    #include "serialconfig.inc"







    /* --------------------------------------------------------------
    // COLOQUE ABAIXO AS OPCOES QUE DEVEM RODAR APOS A CONFIGURACAO
    // DA PORTA SERIAL.
    // -------------------------------------------------------------- */

    /* ------------------------------------------------------------
    // Se tiver menos do que 3 parametros, apenas informa o
    // tipo de chip montado no programador.
    // ------------------------------------------------------------- */
    if(argc < 3) {
      comando[0]='9';
      comando[1]='\r';
      runOptionInfo(comando);
      return 0;
    }

    /* ------------------------------------------------------------------
       OPCOES DE TESTE DE HARDWARE                             27/08/2020
       Essas opcoes nao dependem de identificacao do chip alvo por isso
       devem permanecer nesta posicao do programa.
     -------------------------------------------------------------------
    */

    /* VERIFICACAO DA TENSAO DE ALIMENTACAO DO CIRCUITO PROGRAMADOR */
     if( checkOpt("-vd", argc, argv) == 0 ) {
        comando[0]='7';
        comando[1]='\r';
        if( runOptionVD(comando) == -1) {
          printf("main(): ERRO - NAO PODE VERIFICAR A TENSAO DE ALIMENTACAO\n");
          return -1;
        }
        printf("main(): OPERACAO COMPLETADA COM SUCESSO\n");
        return 0;
    }


    /* VERIFICACAO DA TENSAO DE PROGRAMACAO DO CIRCUITO PROGRAMADOR */
     if( checkOpt("-vp", argc, argv) == 0 ) {
        comando[0]='8';
        comando[1]='\r';
        if( runOptionVP(comando) == -1 ) {
          printf("main(): ERRO - NAO PODE VERIFICAR A TENSAO DE PROGRAMACAO\n");
          return -1;
        }
        printf("main(): OPERACAO COMPLETADA COM SUCESSO\n");
        return 0;
    }

   /* ------------------------------------------------------------------
       OPCOES DE PROGRAMACAO                                  27/08/2020
       Essas opcoes dependem de identificacao do chip alvo por isso
       devem permanecer nesta posicao do programa.
     -------------------------------------------------------------------
   */


    /* DISPLAY DA MEMORIA DE CONFIGURACAO */
    /*if( strcmp(spin_opt, "-dc") == 0 ) { */
    if( checkOpt("-dc", argc, argv) == 0 ) {
        comando[0]='1';
        comando[1]='\r';
        if( runOptionDC(comando) == -1 ) {
           printf("main(): ERRO - NAO PODE LISTAR MEMORIA DE CONFIGURACAO\n");
           return -1;
        }
        printf("main(): OPERACAO COMPLETADA COM SUCESSO\n");
        return 0;
    }


    /* DISPLAY DA MEMORIA EEPROM */
    if( checkOpt("-de", argc, argv) == 0 ) {
        comando[0]='2';
        comando[1]='\r';
        if( runOptionDE(comando) == -1 ) {
           printf("main(): ERRO - NAO PODE LISTAR A EEPROM\n");
           return -1;
        }
        printf("main(): OPERACAO COMPLETADA COM SUCESSO\n");
        return 0;
    }


    /* DISPLAY DA MEMORIA DE PROGRAMA */
    if( checkOpt("-dp", argc, argv) == 0 ) {

        comando[0] = '3';
        comando[1] = '\r';
        if( runOptionDP(comando) == -1 ) {
           printf("main(): ERRO - NAO PODE LISTAR A MEMORIA DE PROGRAMA\n");
           return -1;
        }
        printf("main(): OPERACAO COMPLETADA COM SUCESSO\n");
        return 0;
    }


    /* APAGA MEMORIA EEPROM */
    if( checkOpt("-ee", argc, argv) == 0 ) {
        comando[0] = '4';
        comando[1] = '\r';
        if( runOptionEE(comando) == -1) {
          printf("main(): ERRO AO TENTAR APAGAR A MEMORIA EEPROM\n");
          return -1;
        }
        return 0;

    }


    /* APAGA MEMORIA DE PROGRAMA */
     if( checkOpt("-ep", argc, argv) == 0 ) {
        comando[0] = '5';
        comando[1] = '\r';
        if( runOptionEP(comando) == -1 ) {
           printf("main(): ERRO AO TENTAR APAGAR A MEMORIA DE PROGRAMA\n");
          return -1;
        }
        return 0;

    }


    /* PROGRAMACAO DO MICROCONTROLADOR */
    if(checkOpt("-wp", argc, argv) == 0 ) {
          programfile = getOpt("-wp", argc, argv);
          if( checkOpt("-pic", argc, argv) != 0 ) {
             printf("main(): MICROCONTROLAOR ALVO NAO ESPECIFICADO\n");
             return -1;
          }

          /* Verifica se o chip da opcao -pic e' o mesmo que esta montado no programador */
          char* microctl = getOpt("-pic", argc, argv);

          if( checkChip(microctl) != 0 ) {
             return -1;
          }

          comando[0] = '6';
          comando[1] = '\r';
          if( runOptionWP(comando, microctl) == -1 ) {
             printf("main(): ERRO DURANTE A GRAVACAO DO PROGRAMA\n");
             return -1;
          }
          printf("main(): OPERACAO COMPLETADA COM SUCESSO\n");
          return 0;
    }



    /* INFORMACAO SOBRE O CHIP ALVO (-v --> VERBOSE) */
    if(checkOpt("-v", argc, argv) == 0 ) {
        comando[0]='9';
        comando[1]='\r';
        if( runOptionInfo(comando) == -1 ) {
           printf("main(): INFORMACAO NAO DISPONIVEL\n");
           return -1;
        }
        printf("main(): OPERACAO COMPLETADA COM SUCESSO\n");
        return 0;
    }


     return 0;

}


